@@ -188,7 +188,7 @@
 #line 5 "./parser.yy"
 
 #include "common.h"
-RCSID("$Id: parser.yy,v 1.11 2004/06/30 10:06:11 wojdyr Exp $")
+RCSID("$Id: parser.yy,v 1.12 2005/02/16 21:04:26 wojdyr Exp $")
 #include <string>
 #include <stdlib.h>
 #include <vector>
@@ -1585,8 +1585,8 @@ yyreduce:
   case 15:
 #line 119 "./parser.yy"
     { 
-                                   my_data->load(yyvsp[-1].s.str(), yyvsp[-3].c, ivec, ivec2, yyvsp[-2].i);
-				   my_core->set_view (Rect()); ;}
+                             my_data->load_file(yyvsp[-1].s.str(), yyvsp[-3].c, ivec, ivec2, yyvsp[-2].i);
+			     my_core->set_view (Rect()); ;}
     break;
 
   case 16:
