@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 #include "common.h"
-RCSID ("$Id: sum.cpp,v 1.9 2004/06/16 17:47:53 wojdyr Exp $")
+RCSID ("$Id: sum.cpp,v 1.12 2005/02/24 20:59:38 wojdyr Exp $")
 
 #include "sum.h"
 #include <algorithm>
@@ -106,11 +106,19 @@ void Sum::use_param_a_for_value (const vector<fp>& A) const
 fp Sum::value (fp x) const
     // pre: use_param_a_for_value() called (or not in special cases)
 {
+#ifdef USE_HASHMAP
+    hash_map<fp, fp>::const_iterator hi = buffered_sum.find(x);
+    if (hi != buffered_sum.end())
+        return hi.second;
+#endif
     x += zero_shift(x);
     fp y = 0;
     for(vector<V_f*>::const_iterator i = fvec.begin(); i != fvec.end(); i++)
         if (!cut_tails || (*i)->range_l <= x && x <= (*i)->range_r)
             y += (*i)->compute (x, 0);
+#ifdef USE_HASHMAP
+    buffered_sum[x] = y;
+#endif
     return y;
 }
 
@@ -214,6 +222,30 @@ fp Sum::value_and_add_numeric_deriv (fp x, bool both_sides,
     return y;
 }
 
+// estimate max. value in given range (probe at peak centers and between)
+fp Sum::approx_max(fp x_min, fp x_max)
+{
+    use_param_a_for_value();
+    fp x = x_min;
+    fp y_max = value(x);
+    vector<fp> xx; 
+    for (vector<V_f*>::const_iterator i = fvec.begin(); i != fvec.end(); i++) {
+        fp ctr = (*i)->center();
+        if (x_min < ctr && ctr < x_max)
+            xx.push_back(ctr);
+    }
+    xx.push_back(x_max);
+    sort(xx.begin(), xx.end());
+    for (vector<fp>::const_iterator i = xx.begin(); i != xx.end(); i++) {
+        fp x_between = (x + *i)/2.;
+        x = *i;
+        fp y = max(value(x_between), value(x));
+        if (y > y_max) 
+            y_max = y;
+    }
+    return y_max;
+}
+
 string Sum::general_info() const
 {
     ostringstream os; 
@@ -286,8 +318,18 @@ void Sum::export_as_peaks (std::ostream& os, const vector<int> &peaks) const
         os << "  " << p->type_info()->name; //Type
         os << "  " << p->center() << " " << p->height() << " " << p->area() 
             << " " << p->fwhm() << "  ";
-        for (int i = 0; i < p->type_info()->psize; i++) // a0  a1 ...
-            os << " " << p->get_pag(i).value(parameters->values(), gvec);
+        for (int j = 0; j < p->type_info()->psize; j++) // a0  a1 ...
+            os << " " << p->get_pag(j).value(parameters->values(), gvec);
+        if (refs_to_f(*i)) {
+            os << "  \"";
+            string s = descr_refs_to_f(*i);
+            for (string::const_iterator j = s.begin(); j != s.end(); ++j)
+                if (*j == '"')
+                    os << "\\\"";
+                else
+                    os << *j;
+            os << '"';
+        }
         os << endl;
     }
 }
@@ -519,7 +561,7 @@ int Sum::add_fzg (One_of_fzg fzg, char type, const vector<Pag> &p,
     else if (fzg == gType) gvec.push_back (static_cast<V_g*>(v));
     int n = fzg_size(fzg) - 1;
     if (fzg == fType || fzg == zType) 
-        s_was_changed = true;
+        notify_change();
     verbose ("Added " + V_fzg::full_type(fzg) + S(n));
     v->c_name = V_fzg::short_type(fzg) + S(n);
     v->c_description = desc;
@@ -632,14 +674,14 @@ int Sum::rm_fzg (One_of_fzg fzg, int n, bool silent)
     const V_fzg *ptr = get_fzg (fzg, n);
     if (fzg == fType) {
         fvec.erase (fvec.begin() + n);
-        s_was_changed = true;
+        notify_change();
         for (set<FuncContainer*>::iterator i = f_usage.begin(); 
                 i != f_usage.end(); i++)
             (*i)->synch_after_rm_f (n);
     }
     else if (fzg == zType) {
         zvec.erase (zvec.begin() + n);
-        s_was_changed = true;
+        notify_change();
     }
     else if (fzg == gType) {
         gvec.erase (gvec.begin() + n);
@@ -660,6 +702,14 @@ int Sum::rm_fzg (One_of_fzg fzg, int n, bool silent)
     return 1;
 }
 
+void Sum::notify_change() 
+{ 
+    s_was_changed = true; 
+#ifdef USE_HASHMAP
+    buffered_sum.clear(); 
+#endif
+}
+
 int Sum::rm_all()
 {
     int b_f = fzg_size(fType), b_z = fzg_size(zType), b_g = fzg_size(gType),
@@ -714,11 +764,11 @@ void Sum::change_in_f (int f, vector<int> arg, vector<Pag> np) //experimental
     PagContainer (this, old); //to call recursive_rm() in destructor
 }
 
-/** guess_f() tries to find out what the fit should be for the peak 
- * - much like what happens when you add a peak using the range function. 
+/* guess_f() tries to find out what the fit should be for the peak 
+ * - much like what happens when you add a new peak. 
  * The only difference is that this is done on already existing 
- * peaks - for instance when a nearby peak is moved etc and the user wants 
- * this why estimated again. It changes only these peak parameters, 
+ * peak - for instance when a nearby peak is moved etc and the user wants 
+ * estimate this again. It changes only these peak parameters, 
  * that are simple parameters.
  */
 void Sum::guess_f(int n)
