@@ -1,5 +1,5 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: sum.h,v 1.6 2004/06/16 17:47:53 wojdyr Exp $
+// $Id: sum.h,v 1.8 2005/02/24 20:59:38 wojdyr Exp $
 
 #ifndef SUM__H__
 #define SUM__H__
@@ -8,6 +8,9 @@
 #include <set>
 #include <utility>
 #include <memory>
+#ifdef USE_HASHMAP
+#include <hash_map>
+#endif
 #include "common.h"
 #include "dotset.h"
 #include "pag.h"
@@ -36,7 +39,7 @@ public:
     fp value_and_put_deriv (fp x, std::vector<fp>& dy_da) const;
     fp value_and_add_numeric_deriv (fp x, bool both_sides, 
                                     std::vector<fp>& dy_da) const;
-    fp get_ffunc_deriv (int f, fp x, std::vector<fp>& dy_da);
+    fp approx_max(fp x_min, fp x_max);
     std::string general_info() const;
     std::string sum_full_formula (const std::vector<fp>& localA = fp_v0) const;
     std::string sum_of_peaks_formula (const std::vector<int>& peaks, 
@@ -111,7 +114,11 @@ private:
     mutable std::vector<fp> av_numder;
     std::set<FuncContainer*> f_usage;
     std::set<PagContainer*> vp_usage;
+#ifdef USE_HASHMAP
+    std::hash_map<fp, fp> sum_buffer
+#endif
 
+    void notify_change();
     std::string format_der_output(const std::vector<fp>& df_s, 
                    const std::vector<fp>& df_n1, const std::vector<fp>& df_n2);
     void export_fzg_as_script (std::ostream& os, One_of_fzg fzg) const;
