@@ -10,7 +10,7 @@
 #endif
 
 #include "common.h"
-RCSID ("$Id: wx_gui.cpp,v 1.25 2004/11/12 17:16:31 wojdyr Exp $")
+RCSID ("$Id: wx_gui.cpp,v 1.32 2005/03/04 15:13:03 wojdyr Exp $")
 
 #include <wx/laywin.h>
 #include <wx/splitter.h>
@@ -28,6 +28,7 @@ RCSID ("$Id: wx_gui.cpp,v 1.25 2004/11/12 17:16:31 wojdyr Exp $")
 #include <wx/printdlg.h>
 #include <wx/image.h>
 #include <wx/config.h>
+#include <wx/msgout.h>
 #include <algorithm>
 #include <locale.h>
 #include <string.h>
@@ -87,14 +88,14 @@ char* about_html =
 "<html> <body bgcolor=white text=white>                                  "
 "  <table cellspacing=3 cellpadding=4 width=100%>                        "
 "   <tr bgcolor=#101010 align=center> <td>                               "
-"      <h1> fityk </h1>                                                  "
-"      <h3> ver. " VERSION "</h3>                                        "
-"      <h6> $Date: 2004/11/12 17:16:31 $</h6>                            "
+"      <h1> fityk " VERSION "</h1>                                       "
+"      <h6> powered by " wxVERSION_STRING "</h6>                         "
+"      <p><font size=-2>$Date: 2005/03/04 15:13:03 $</font></p>          "
 "   </td> </tr>                                                          "
 "   <tr> <td bgcolor=#73A183>                                            "
 "     <font color=black>                                                 "
 "      <b><font size=+1> <p align = center>                              "
-"       Copyright (C) 2001 - 2004 Marcin Wojdyr                          "
+"       Copyright (C) 2001 - 2005 Marcin Wojdyr                          "
 "      </font></b><p>                                                    "
 "      <font size=-1>                                                    "
 "       This program is free software; you can redistribute it           "
@@ -114,13 +115,14 @@ FFrame *frame = NULL;
 std::vector<fp> params4plot;  //TODO move it to MainPlot or other class
 
 enum {
-    ID_QUIT             = 44001,
+    ID_QUIT            = 24001 ,
     ID_H_MANUAL                ,
     ID_H_TIP                   ,
     ID_D_LOAD                  ,
     ID_D_XLOAD                 ,
-    ID_D_RECENT         = 44010, //and next ones
-    ID_D_INFO           = 44130,
+    ID_D_RECENT        = 24010 , //and next ones
+    ID_D_INFO          = 24130 ,
+    ID_D_EDITOR                ,
     ID_D_DEVIATION             ,
     ID_D_RANGE                 ,
     ID_D_B_INFO                ,
@@ -144,8 +146,8 @@ enum {
     ID_F_CONTINUE              ,
     ID_F_INFO                  ,
     ID_F_SET                   ,
-    ID_F_M             = 44160 , // and a few next IDs
-    ID_C_WAVELENGTH    = 44171 ,
+    ID_F_M             = 24160 , // and a few next IDs
+    ID_C_WAVELENGTH    = 24171 ,
     ID_C_ADD                   ,
     ID_C_INFO                  ,
     ID_C_REMOVE                ,
@@ -167,8 +169,8 @@ enum {
     ID_G_M_BG                  ,
     ID_G_M_ADD                 ,
     ID_G_M_PEAK                ,
-    ID_G_M_PEAK_N      = 44220 ,
-    ID_G_SHOW          = 44260 ,
+    ID_G_M_PEAK_N      = 24220 ,
+    ID_G_SHOW          = 24260 ,
     ID_G_S_TOOLBAR             ,
     ID_G_S_STATBAR             ,
     ID_G_S_DPANE               ,
@@ -180,8 +182,8 @@ enum {
     ID_G_V_VERT                ,
     ID_G_V_SCROLL_L            ,
     ID_G_V_SCROLL_R            ,
-    ID_G_V_ZOOM_PREV   = 44302 ,
-    ID_G_LCONF1        = 44342 ,
+    ID_G_V_ZOOM_PREV   = 24302 ,
+    ID_G_LCONF1        = 24342 ,
     ID_G_LCONF2                ,
     ID_G_LCONFB                ,
     ID_G_SCONF                 ,
@@ -215,6 +217,8 @@ static const wxCmdLineEntryDesc cmdLineDesc[] = {
     { wxCMD_LINE_SWITCH, "q", "quiet",   "be quiet", wxCMD_LINE_VAL_NONE, 0 },
     { wxCMD_LINE_OPTION, "o", "output", "output file", wxCMD_LINE_VAL_NONE, 0 },
     */
+    { wxCMD_LINE_SWITCH, "V", "version", 
+              "output version information and exit", wxCMD_LINE_VAL_NONE, 0 },
     { wxCMD_LINE_OPTION, "c", "cmd", "script passed in as string", 
                                                    wxCMD_LINE_VAL_STRING, 0 },
     { wxCMD_LINE_PARAM,  0, 0, "script or data file", wxCMD_LINE_VAL_STRING, 
@@ -232,8 +236,12 @@ bool FApp::OnInit(void)
     wxCmdLineParser cmdLineParser(cmdLineDesc, argc, argv);
     if (cmdLineParser.Parse(false) != 0) {
         cmdLineParser.Usage();
-        return false;
+        return false; //false = exit the application
     }
+    else if (cmdLineParser.Found("V")) {
+        wxMessageOutput::Get()->Printf("fityk version " VERSION "\n");
+        return false; //false = exit the application
+    } //the rest of options will be processed in process_argv()
 
     AL = new ApplicationLogic; 
 
@@ -248,7 +256,7 @@ bool FApp::OnInit(void)
     alt_conf_filename = pre + "alt-config";
     wxConfigBase *config = new wxConfig("", "", pre+"wxoptions", "", 
                                         wxCONFIG_USE_LOCAL_FILE);
-    wxConfig::Set (config);
+    wxConfig::Set(config);
 
     // Create the main frame window
     frame = new FFrame(NULL, -1, app_name, wxDEFAULT_FRAME_STYLE);
@@ -330,20 +338,23 @@ void FApp::process_argv(wxCmdLineParser &cmdLineParser)
     wxString cmd;
     if (cmdLineParser.Found("c", &cmd))
         getUI()->execAndLogCmd(cmd.c_str());
-    int data_flag = false;
+    //the rest of parameters/arguments are scripts and/or data files
+    int data_counter = 0; //number of data files
     for (unsigned int i = 0; i < cmdLineParser.GetParamCount(); i++) {
         string par = cmdLineParser.GetParam(i).c_str();
         if (is_fityk_script(par))
             getUI()->execScript(par);
         else {
-            if (!data_flag) {
+            //if there are multiple data files specified at command line,
+            //open each as separate dataset in the same plot
+            if (data_counter == 0) {
                 getUI()->execAndLogCmd("d.load '" + par + "'");
-                data_flag = true;
             }
             else {
                 getUI()->execAndLogCmd("d.activate ::* ; d.load '" + par + "'");
                 frame->SwitchDPane(true);
             }
+            data_counter++;
         }
     }
 }
@@ -357,6 +368,7 @@ BEGIN_EVENT_TABLE(FFrame, wxFrame)
     EVT_MENU (ID_D_LOAD,        FFrame::OnDLoad)   
     EVT_MENU (ID_D_XLOAD,       FFrame::OnDXLoad)   
     EVT_MENU_RANGE (ID_D_RECENT+1, ID_D_RECENT+100, FFrame::OnDRecent)
+    EVT_MENU (ID_D_EDITOR,      FFrame::OnDEditor)
     EVT_MENU (ID_D_INFO,        FFrame::OnDInfo)
     EVT_MENU (ID_D_DEVIATION  , FFrame::OnDDeviation)  
     EVT_MENU (ID_D_RANGE,       FFrame::OnDRange)
@@ -446,7 +458,7 @@ FFrame::FFrame(wxWindow *parent, const wxWindowID id, const wxString& title,
                  const long style)
     : wxFrame(parent, id, title, wxDefaultPosition, wxDefaultSize, style), 
       main_pane(0), data_pane(0), status_bar(0), 
-      peak_type_nr(0), toolbar(0), dxload_dialog(0), 
+      peak_type_nr(0), toolbar(0), dxload_dialog(0), data_editor(0),
       print_data(new wxPrintData), page_setup_data(new wxPageSetupData),
 #ifdef __WXMSW__
       help()
@@ -636,6 +648,7 @@ void FFrame::set_menubar()
     data_menu->Append(ID_D_RECENT, "Recent &Files", data_menu_recent); 
     data_menu->AppendSeparator();
 
+    data_menu->Append (ID_D_EDITOR,   "&Viewer", "Open data viewer");
     data_menu->Append (ID_D_INFO,     "&Info", "Info about loaded data");
     data_menu->Append (ID_D_DEVIATION  , "Std. &Dev.", 
                                             "Change errors for data points");
@@ -782,13 +795,13 @@ void FFrame::set_menubar()
     session_menu->Append (ID_O_WAIT,    "&Wait", "Just wait (rather useless)");
     session_menu->AppendSeparator();
     session_menu->Append (ID_O_SET,     "&Settings", "Preferences and options");
+    session_menu->AppendSeparator();
+    session_menu->Append(ID_QUIT, "&Quit", "Exit the program");
 
     wxMenu *help_menu = new wxMenu;
     help_menu->Append(ID_H_MANUAL, "&Manual", "User's Manual");
     help_menu->Append(ID_H_TIP, "&Tip of the day", "Show tip of the day");
     help_menu->Append(wxID_ABOUT, "&About...", "Show about dialog");
-    //help_menu->AppendSeparator();
-    //help_menu->Append(ID_QUIT, "&Exit", "Exit the program");
 
     wxMenuBar *menu_bar = new wxMenuBar(wxMENU_TEAROFF);
     menu_bar->Append (session_menu, "S&ession" );
@@ -930,6 +943,17 @@ void FFrame::OnDRecent (wxCommandEvent& event)
     add_recent_data_file(s);
 }
 
+void FFrame::OnDEditor (wxCommandEvent& WXUNUSED(event))
+{
+    if (!data_editor) {
+        data_editor = new DataEditorDlg(this, -1, my_data);
+        data_editor->Show();
+    }
+    else {
+        data_editor->update_data(my_data);
+    }
+}
+
 void FFrame::OnDInfo (wxCommandEvent& WXUNUSED(event))
 {
     exec_command ("d.info");
@@ -972,12 +996,17 @@ void FFrame::OnDCalibrate (wxCommandEvent& WXUNUSED(event))
 
 void FFrame::OnDExport (wxCommandEvent& WXUNUSED(event))
 {
-    wxFileDialog fdlg (frame, "Export data to file", "", "",
+    static int filter_idx = 0;
+    static wxString dir = ".";
+    wxFileDialog fdlg (frame, "Export data to file", dir, "",
                        "x y data (*.dat)|*.dat;*.DAT"
                        "|fityk file (*.fit)|*.fit;*.FIT",
                        wxSAVE | wxOVERWRITE_PROMPT);
+    fdlg.SetFilterIndex(filter_idx);
     if (fdlg.ShowModal() == wxID_OK) 
         exec_command (("d.export '" + fdlg.GetPath() + "'").c_str());
+    filter_idx = fdlg.GetFilterIndex();
+    dir = fdlg.GetDirectory();
 }
 
 void FFrame::OnDSet (wxCommandEvent& WXUNUSED(event))
@@ -1034,15 +1063,21 @@ void FFrame::OnSValue        (wxCommandEvent& WXUNUSED(event))
           
 void FFrame::OnSExport       (wxCommandEvent& WXUNUSED(event))
 {
-    wxFileDialog fdlg (frame, "Export curve to file", "", "",
-                       "x y data (*.dat)|*.dat;*.DAT"
+    static int filter_idx = 0;
+    static wxString dir = ".";
+    wxString name = wxFileName(my_data->get_filename().c_str()).GetName();
+    wxFileDialog fdlg (frame, "Export curve to file", dir, name,
+                       "parameters of peaks (*.peaks)|*.peaks"
+                       "|x y data (*.dat)|*.dat;*.DAT"
                        "|XFIT peak listing (*xfit.txt)|*xfit.txt;*XFIT.TXT" 
                        "|mathematic formula (*.formula)|*.formula"
-                       "|parameters of peaks (*.peaks)|*.peaks"
                        "|fityk file (*.fit)|*.fit;*.FIT",
                        wxSAVE | wxOVERWRITE_PROMPT);
+    fdlg.SetFilterIndex(filter_idx);
     if (fdlg.ShowModal() == wxID_OK) 
         exec_command (("s.export '" + fdlg.GetPath() + "'").c_str());
+    filter_idx = fdlg.GetFilterIndex();
+    dir = fdlg.GetDirectory();
 }
            
 void FFrame::OnSSet          (wxCommandEvent& WXUNUSED(event))
@@ -1547,9 +1582,9 @@ void FFrame::output_text(OutputStyle style, const string& str)
     io_pane->append_text(style, str.c_str());
 }
 
-void FFrame::refresh_plots(bool update)
+void FFrame::refresh_plots(bool refresh, bool update)
 {
-    plot_pane->refresh_plots(update);
+    plot_pane->refresh_plots(refresh, update);
 }
 
 void FFrame::draw_crosshair(int X, int Y)
@@ -1593,7 +1628,7 @@ void add_peak(fp height, fp ctr, fp hwhm)
     string stat = "Height: " + S(height) + " Ctr: " + S(center) 
                    + " HWHM: " + S(hwhm);
 //return cmd, stat
-    frame->SetStatusText (stat.c_str());
+    frame->set_status_text(stat.c_str());
     exec_command (cmd);
 }
 
