@@ -10,7 +10,7 @@
 #endif
 
 #include "common.h"
-RCSID ("$Id: wx_pane.cpp,v 1.15 2004/10/29 15:05:12 wojdyr Exp $")
+RCSID ("$Id: wx_pane.cpp,v 1.20 2005/03/04 14:28:47 wojdyr Exp $")
 
 #include <wx/colordlg.h>
 #include <wx/fontdlg.h>
@@ -26,10 +26,7 @@ RCSID ("$Id: wx_pane.cpp,v 1.15 2004/10/29 15:05:12 wojdyr Exp $")
 using namespace std;
 
 enum { 
-    ID_COMBO            = 47001,
-    ID_OUTPUT_TEXT             ,
-
-    ID_OUTPUT_C_BG             ,
+    ID_OUTPUT_C_BG      = 27001,
     ID_OUTPUT_C_IN             ,
     ID_OUTPUT_C_OU             ,
     ID_OUTPUT_C_QT             ,
@@ -109,12 +106,14 @@ void PlotPane::read_settings(wxConfigBase *cf)
         aux_plot[i]->read_settings(cf);
 }
 
-void PlotPane::refresh_plots(bool update)
+void PlotPane::refresh_plots(bool refresh, bool update)
 {
     vector<FPlot*> vp = get_visible_plots();
     for (vector<FPlot*>::const_iterator i = vp.begin(); i != vp.end(); ++i) {
-        (*i)->Refresh(false);
-        if (update) (*i)->Update();
+        if (refresh)
+            (*i)->Refresh(false);
+        if (update) 
+            (*i)->Update();
     }
 }
 
@@ -218,22 +217,17 @@ BEGIN_EVENT_TABLE(IOPane, wxPanel)
 END_EVENT_TABLE()
 
 IOPane::IOPane(wxWindow *parent, wxWindowID id)
-    : wxPanel(parent, id), output_win(0), input_combo(0)
+    : wxPanel(parent, id), output_win(0), input_field(0)
 {
     wxBoxSizer *io_sizer = new wxBoxSizer (wxVERTICAL);
 
-    // wxTextCtrl which displays output of commands
-    output_win = new OutputWin (this, ID_OUTPUT_TEXT);
+    output_win = new OutputWin (this, -1);
     io_sizer->Add (output_win, 1, wxEXPAND);
 
-    // FCombo - wxComboBox used for user keybord input
-    //wxString input_choices[] = { /*"help"*/ };
-    input_combo = new FCombo (this, ID_COMBO, "",
-                               wxDefaultPosition, wxDefaultSize, 
-                               0, 0,//input_choices, 
-                               wxCB_DROPDOWN|wxWANTS_CHARS|
-                               wxTE_PROCESS_ENTER|wxTE_PROCESS_TAB);
-    io_sizer->Add (input_combo, 0, wxEXPAND);
+    input_field = new InputField (this, -1, "",
+                            wxDefaultPosition, wxDefaultSize, 
+                            wxWANTS_CHARS|wxTE_PROCESS_ENTER|wxTE_PROCESS_TAB);
+    io_sizer->Add (input_field, 0, wxEXPAND);
 
     SetAutoLayout (true);
     SetSizer (io_sizer);
@@ -477,8 +471,9 @@ void OutputWin::read_settings(wxConfigBase *cf)
     text_color[os_input] = read_color_from_config(cf, "input", 
                                                      wxColour(0, 200, 0));
     bg_color = read_color_from_config(cf, "bg", wxColour(20, 20, 20));
-
-    SetDefaultStyle (wxTextAttr(text_color[os_quot], bg_color));
+    cf->SetPath("/OutputWin");
+    wxFont font = read_font_from_config(cf, "font", wxNullFont);
+    SetDefaultStyle (wxTextAttr(text_color[os_quot], bg_color, font));
 
     // this "if" is needed on GTK 1.2 (I don't know why)
     // if it is called before window is shown, it doesn't work 
@@ -496,12 +491,18 @@ void OutputWin::save_settings(wxConfigBase *cf) const
     write_color_to_config (cf, "quot", text_color[os_quot]); 
     write_color_to_config (cf, "input", text_color[os_input]); 
     write_color_to_config (cf, "bg", bg_color); 
+    cf->SetPath("/OutputWin");
+    write_font_to_config (cf, "font", GetDefaultStyle().GetFont());
 }
 
 void OutputWin::append_text (OutputStyle style, const wxString& str)
 {
     SetDefaultStyle (wxTextAttr (text_color[style]));
     AppendText (str);
+    ShowPosition(GetLastPosition());
+    //TODO TextCtrl is not refreshed when program is busy
+    //I think it used to work with older versions of wx
+    //Refresh() and Update() here don't help
 }
 
 void OutputWin::OnPopupColor (wxCommandEvent& event)
@@ -536,7 +537,7 @@ void OutputWin::OnPopupFont (wxCommandEvent& WXUNUSED(event))
 {
     wxFontData data; 
     data.SetInitialFont (GetDefaultStyle().GetFont());
-    wxFontDialog dlg (this, &data);
+    wxFontDialog dlg (frame, &data);
     int r = dlg.ShowModal();
     if (r == wxID_OK) {
         wxFont f = dlg.GetFontData().GetChosenFont();
@@ -581,41 +582,62 @@ void OutputWin::OnKeyDown (wxKeyEvent& event)
 }
 
 //===============================================================
-//                            combo
+//                            input field
 //===============================================================
 
-BEGIN_EVENT_TABLE(FCombo, wxComboBox)
-    EVT_KEY_DOWN (FCombo::OnKeyDown)
+BEGIN_EVENT_TABLE(InputField, wxTextCtrl)
+    EVT_KEY_DOWN (InputField::OnKeyDown)
 END_EVENT_TABLE()
 
-void FCombo::OnKeyDown (wxKeyEvent& event)
+void InputField::OnKeyDown (wxKeyEvent& event)
 {
-    const int n_list_items = 15;
-    if (event.m_keyCode == WXK_RETURN || event.m_keyCode == WXK_NUMPAD_ENTER) {
-        wxString s = GetValue().Trim();
-        if (s.IsEmpty())
-            return;
-        frame->SetStatusText (s);
-        // changing drop-down list
-        vector<wxString> list;
-        list.push_back(s);
-        int n = std::min (n_list_items, GetCount() + 1);
-        for (int i = 0; i < n - 1; i++)
-            list.push_back(GetString(i));
-        Clear();
-        for (vector<wxString>::iterator i = list.begin(); i != list.end(); i++)
-            Append (*i);
-        SetValue("");
-
-        //displaying and executing command
-        exec_command (s.c_str());
+    switch (event.m_keyCode) {
+        case WXK_RETURN:
+        case WXK_NUMPAD_ENTER:
+            {
+            wxString s = GetValue().Trim();
+            if (s.IsEmpty())
+                return;
+            frame->set_status_text(s);
+            history.insert(++history.begin(), s);
+            h_pos = history.begin();
+            Clear();
+            exec_command (s.c_str()); //displaying and executing command
+            }
+            break;
+        case WXK_TAB:
+            {
+            IOPane *parent = static_cast<IOPane*>(GetParent());//to not use RTTI
+            parent->focus_output();
+            }
+            break;
+        case WXK_UP:
+        case WXK_NUMPAD_UP:
+            if (h_pos == --history.end())
+                return;
+            ++h_pos;
+            SetValue(*h_pos);
+            break;
+        case WXK_DOWN:
+        case WXK_NUMPAD_DOWN:
+            if (h_pos == history.begin())
+                return;
+            --h_pos;
+            SetValue(*h_pos);
+            break;
+        case WXK_PAGEUP:
+        case WXK_NUMPAD_PAGEUP:
+            h_pos == --history.end();
+            SetValue(*h_pos);
+            break;
+        case WXK_PAGEDOWN:
+        case WXK_NUMPAD_PAGEDOWN:
+            h_pos == history.begin();
+            SetValue(*h_pos);
+            break;
+        default:
+            event.Skip();
     }
-    else if (event.m_keyCode == WXK_TAB) {
-        IOPane *parent = static_cast<IOPane*>(GetParent()); //to not use RTTI
-        parent->focus_output();
-    }
-    else
-        event.Skip();
 }
 
 //===============================================================
@@ -680,6 +702,41 @@ void write_color_to_config (wxConfigBase *config, const wxString& key,
     config->Write (key + "/Blue", value.Blue());
 }
 
+wxFont read_font_from_config (const wxConfigBase *config, const wxString& key,
+                              const wxFont& default_value)
+{
+    if (!default_value.Ok()) {
+        if (config->HasEntry(key+"/pointSize")
+              && config->HasEntry(key+"/family")
+              && config->HasEntry(key+"/style")
+              && config->HasEntry(key+"/weight")
+              && config->HasEntry(key+"/faceName"))
+            return wxFont (config->Read(key+"/pointSize", 0L),
+                           config->Read(key+"/family", 0L),
+                           config->Read(key+"/style", 0L),
+                           config->Read(key+"/weight", 0L),
+                           false, //underline
+                           config->Read(key+"/faceName", ""));
+        else
+            return wxNullFont;
+    }
+    return wxFont (config->Read(key+"/pointSize", default_value.GetPointSize()),
+                   config->Read(key+"/family", default_value.GetFamily()),
+                   config->Read(key+"/style", default_value.GetStyle()),
+                   config->Read(key+"/weight", default_value.GetWeight()),
+                   false, //underline
+                   config->Read(key+"/faceName", default_value.GetFaceName()));
+}
+
+void write_font_to_config (wxConfigBase *config, const wxString& key,
+                           const wxFont& value)
+{
+    config->Write (key + "/pointSize", value.GetPointSize());
+    config->Write (key + "/family", value.GetFamily());
+    config->Write (key + "/style", value.GetStyle());
+    config->Write (key + "/weight", value.GetWeight());
+    config->Write (key + "/faceName", value.GetFaceName());
+}
 
 //===============================================================
 //                            ProportionalSplitter
