@@ -10,13 +10,15 @@
 #endif
 
 #include "common.h"
-RCSID ("$Id: wx_plot.cpp,v 1.18 2004/10/28 20:12:30 wojdyr Exp $")
+RCSID ("$Id: wx_plot.cpp,v 1.23 2005/02/23 20:41:02 wojdyr Exp $")
 
 #include <wx/laywin.h>
 #include <wx/sashwin.h>
 #include <wx/colordlg.h>
 #include <wx/gdicmn.h>
 #include <wx/statline.h>
+#include <wx/fontdlg.h>
+#include <wx/textdlg.h>
 #include <algorithm>
 #include <numeric>
 #include <vector>
@@ -40,34 +42,39 @@ RCSID ("$Id: wx_plot.cpp,v 1.18 2004/10/28 20:12:30 wojdyr Exp $")
 using namespace std;
 
 enum {
-    ID_plot_popup_za                = 45001, 
-    ID_plot_popup_data              = 45011,
+    ID_plot_popup_za                = 25001, 
+    ID_plot_popup_data              = 25011,
     ID_plot_popup_sum                      ,
     ID_plot_popup_phase                    ,
     ID_plot_popup_peak                     ,
+    ID_plot_popup_plabels                  ,
     ID_plot_popup_xaxis                    ,
     ID_plot_popup_tics                     ,
+    ID_plot_popup_smooth                   ,
 
     ID_plot_popup_c_background             ,
     ID_plot_popup_c_active_data            ,
     ID_plot_popup_c_inactive_data          ,
     ID_plot_popup_c_sum                    ,
     ID_plot_popup_c_xaxis                  ,
-    ID_plot_popup_c_phase_0         = 45100,
-    ID_plot_popup_c_peak_0          = 45140,
-    ID_plot_popup_c_phase           = 45190,
+    ID_plot_popup_c_phase_0         = 25100,
+    ID_plot_popup_c_peak_0          = 25140,
+    ID_plot_popup_c_phase           = 25190,
     ID_plot_popup_c_peak                   ,
     ID_plot_popup_c_inv                    ,
 
-    ID_plot_popup_buffer                   ,
-    ID_plot_popup_pt_size           = 45210,// and next 10 ,
-    ID_peak_popup_info              = 45250,
+    ID_plot_popup_m_plabel                 ,
+    ID_plot_popup_m_plfont                 ,
+    ID_plot_popup_m_tfont                  ,
+
+    ID_plot_popup_pt_size           = 25210,// and next 10 ,
+    ID_peak_popup_info              = 25250,
     ID_peak_popup_del                      ,
     ID_peak_popup_tree                     ,
     ID_peak_popup_guess                    ,
 
-    ID_aux_popup_plot_0            = 45310,
-    ID_aux_popup_c_background      = 45340,
+    ID_aux_popup_plot_0            = 25310,
+    ID_aux_popup_c_background      = 25340,
     ID_aux_popup_c_active_data            ,
     ID_aux_popup_c_inactive_data          ,
     ID_aux_popup_c_axis                   ,
@@ -131,29 +138,10 @@ bool FPlot::vert_line_following_cursor (Mouse_act_enum ma, int x, int x0)
 BEGIN_EVENT_TABLE(FPlot, wxPanel)
 END_EVENT_TABLE()
 
-//-------------------------------------
-//optimization: remembering sum values
 
-const fp initial_buffered_value = -1.234565;
-
-vector<fp> buffered_sum;
-
-void clear_buffered_sum()
+inline fp sum_value(vector<Point>::const_iterator pt)
 {
-    int size = my_data->points().size();
-    if (size) {
-        buffered_sum.resize (size);
-        fill (buffered_sum.begin(), buffered_sum.end(), initial_buffered_value);
-    }
-}
-
-fp sum_value(vector<Point>::const_iterator pt)
-{
-    vector<fp>::iterator y = buffered_sum.begin() 
-                                            + (pt - my_data->points().begin());
-    if (*y == initial_buffered_value)
-        *y = my_sum->value(pt->x);
-    return *y;
+    return my_sum->value(pt->x);
 }
 
 //===============================================================
@@ -172,15 +160,16 @@ BEGIN_EVENT_TABLE(MainPlot, FPlot)
     EVT_MIDDLE_UP (       MainPlot::OnButtonUp)
     EVT_KEY_DOWN   (      MainPlot::OnKeyDown)
     EVT_MENU (ID_plot_popup_za,     MainPlot::OnZoomAll)
-    EVT_MENU_RANGE (ID_plot_popup_data, ID_plot_popup_tics,  
+    EVT_MENU_RANGE (ID_plot_popup_data, ID_plot_popup_smooth,  
                                     MainPlot::OnPopupShowXX)
-    EVT_MENU (ID_plot_popup_buffer, MainPlot::OnPopupBuffer)
-
     EVT_MENU_RANGE (ID_plot_popup_c_background, ID_plot_popup_c_phase - 1, 
                                     MainPlot::OnPopupColor)
     EVT_MENU_RANGE (ID_plot_popup_pt_size, ID_plot_popup_pt_size + max_radius, 
                                     MainPlot::OnPopupRadius)
     EVT_MENU (ID_plot_popup_c_inv,  MainPlot::OnInvertColors)
+    EVT_MENU (ID_plot_popup_m_plabel,MainPlot::OnPeakLabel)
+    EVT_MENU (ID_plot_popup_m_plfont,MainPlot::OnPlabelFont)
+    EVT_MENU (ID_plot_popup_m_tfont,MainPlot::OnTicsFont)
     EVT_MENU (ID_peak_popup_info,   MainPlot::OnPeakInfo)
     EVT_MENU (ID_peak_popup_del,    MainPlot::OnPeakDelete)
     EVT_MENU (ID_peak_popup_tree,   MainPlot::OnPeakShowTree)
@@ -220,7 +209,7 @@ void MainPlot::Draw(wxDC &dc)
 
     if (!params4plot.empty() && size(params4plot) != my_sum->pars()->count_a()){
         params4plot = my_sum->pars()->values();
-        frame->SetStatusText ("Number of parameters changed.");
+        frame->set_status_text("Number of parameters changed.");
     }
     my_sum->use_param_a_for_value (params4plot);
     prepare_peaktops();
@@ -252,37 +241,26 @@ void MainPlot::Draw(wxDC &dc)
 
     vector<Point>::const_iterator f =my_data->get_point_at(my_core->view.left),
                                 l = my_data->get_point_at(my_core->view.right);
+    if (l != my_data->points().end())
+        ++l;
+
+    if (peaks_visible)
+        draw_peaks (dc, f, l);
+    if (phases_visible)
+        draw_phases (dc, f, l);
+    if (sum_visible)
+        draw_sum (dc, f, l);
     if (x_axis_visible) 
         draw_x_axis (dc, f, l);
 
-/*
-    if (shared.buffer_enabled) {
-        //TODO
-        buffer_peaks (f, l);
-        if (sum_visible)
-            ;//draw_sum_from_buffer (dc);
-        if (phases_visible)
-            ;//draw_phases_from_buffer (dc);
-        if (peaks_visible)
-            ;//draw_peaks_from_buffer (dc);
-        
-    }
-    else 
-*/
-    {
-        if (sum_visible)
-            draw_sum (dc, f, l);
-        if (phases_visible)
-            draw_phases (dc, f, l);
-        if (peaks_visible)
-            draw_peaks (dc, f, l);
-    }
-
+    if (visible_peaktops(mode)) 
+        draw_peaktops(dc); 
     if (mode == mmd_bg) {
         draw_background_points(dc); 
     }
-    else if (visible_peaktops(mode)) {
-        draw_peaktops(dc); 
+    else {
+        if (plabels_visible)
+            draw_plabels(dc);
     }
 }
 
@@ -306,7 +284,7 @@ void MainPlot::draw_x_axis (wxDC& dc, vector<Point>::const_iterator first,
                 dc.DrawLine (X_, Y_, X, Y); 
         }
     }
-    else
+    else 
         dc.DrawLine (0, y2Y(0), GetClientSize().GetWidth(), y2Y(0));
 }
 
@@ -315,8 +293,10 @@ void FPlot::draw_tics (wxDC& dc, const Rect &v,
                           const int x_tic_size, const int y_tic_size)
 {
     dc.SetPen (xAxisPen);
-    dc.SetFont(*wxSMALL_FONT);
+    dc.SetFont(ticsFont);
     dc.SetTextForeground(xAxisPen.GetColour());
+
+    // x axis tics
     fp x_tic_step = scale_tics_step(v.left, v.right, x_max_tics);
     for (fp x = x_tic_step * ceil(v.left / x_tic_step); x < v.right; 
             x += x_tic_step) {
@@ -328,6 +308,8 @@ void FPlot::draw_tics (wxDC& dc, const Rect &v,
         dc.GetTextExtent (label, &w, &h);
         dc.DrawText (label, X - w/2, Y + 1);
     }
+
+    // y axis tics
     fp y_tic_step = scale_tics_step(v.bottom, v.top, y_max_tics);
     for (fp y = y_tic_step * ceil(v.bottom / y_tic_step); y < v.top; 
             y += y_tic_step) {
@@ -461,41 +443,53 @@ void MainPlot::buffer_peaks (vector<Point>::const_iterator /*first*/,
     */
 }
 
-void MainPlot::draw_sum (wxDC& dc, vector<Point>::const_iterator first,
-                                   vector<Point>::const_iterator last)
+
+void MainPlot::draw_sum(wxDC& dc, vector<Point>::const_iterator first,
+                                  vector<Point>::const_iterator last)
 {
     dc.SetPen (sumPen);
-    int X = 0, Y = 0;
-    int Y_zero = y2Y(0);
+    int X = -1, Y = -1;
     for (vector<Point>::const_iterator i = first; i < last; i++) {
         int X_ = X, Y_ = Y;
         X = x2X(i->x);
-        Y = y2Y (my_core->plus_background ? sum_value(i) +  i->get_bg()
-                                           : sum_value(i));
-        if (i != first 
-                && (X != X_ || Y != Y_) 
-                && (Y_ != Y_zero || Y != Y_zero //not drawing on x axis
-                    || my_core->plus_background))
+        Y = y2Y(my_sum->value(i->x) 
+                + (my_core->plus_background && smooth ? i->get_bg() : 0));
+        if (smooth)
+            while (X_ < X-1) {
+                ++X_;
+                int Y_p = Y_;
+                Y_ = y2Y(my_sum->value(X2x(X_)));
+                if (X_ > 0)
+                    dc.DrawLine (X_-1, Y_p, X_, Y_); 
+            }
+        if (X_ >= 0 && (X != X_ || Y != Y_)) 
             dc.DrawLine (X_, Y_, X, Y); 
     }
 }
 
+
 void MainPlot::draw_phases (wxDC& dc, vector<Point>::const_iterator first,
                                       vector<Point>::const_iterator last)
+                            
 {
 #ifdef USE_XTAL
     for (int k = 0; k < my_crystal->get_nr_of_phases(); k++) {
         dc.SetPen (phasePen[k % max_phase_pens]);
-        vector<int>peaks = my_crystal->get_funcs_in_phase(k);
-        int X = 0, Y = 0;
-        int Y_zero = y2Y(0);
+        vector<int> peaks = my_crystal->get_funcs_in_phase(k);
+        int X = -1, Y = -1;
         for (vector<Point>::const_iterator i = first; i < last; i++) {
             int X_ = X, Y_ = Y;
-            X = x2X (i->x);
+            X = x2X(i->x);
             Y = y2Y(my_sum->funcs_value (peaks, i->x));
-            if (i != first 
-                    && (X != X_ || Y != Y_) 
-                    && (Y_ != Y_zero || Y != Y_zero)) //not drawing on x axis
+            if (smooth)
+                while (X_ < X-1) {
+                    ++X_;
+                    int Y_p = Y_;
+                    Y_ = y2Y(my_sum->funcs_value (peaks, X2x(X_)));
+                    if (X_ > 0)
+                        dc.DrawLine (X_-1, Y_p, X_, Y_); 
+                }
+            if (X_ >= 0 && (X != X_ || Y != Y_)) 
                 dc.DrawLine (X_, Y_, X, Y); 
         }
     }
@@ -507,16 +501,21 @@ void MainPlot::draw_peaks (wxDC& dc, vector<Point>::const_iterator first,
 {
     for (int k = 0; k < my_sum->fzg_size(fType); k++) {
         dc.SetPen (peakPen[k % max_peak_pens]);
-        int X = x2X (first->x), 
-            Y = y2Y (my_sum->f_value(first->x, k));
-        int Y_zero = y2Y(0);
-        for (vector<Point>::const_iterator i = first + 1; i < last; i++) {
+        int X = -1, Y = -1;
+        for (vector<Point>::const_iterator i = first; i < last; i++) {
             int X_ = X, Y_ = Y;
-            X = x2X (i->x);
-            Y = y2Y (my_sum->f_value (i->x, k));
-            if (Y_ != Y_zero || Y != Y_zero //not drawing on x axis
-                    && (X != X_ || Y != Y_))
-                dc.DrawLine (X_, Y_, X, Y );
+            X = x2X(i->x);
+            Y = y2Y(my_sum->f_value (i->x, k));
+            if (smooth)
+                while (X_ < X-1) {
+                    ++X_;
+                    int Y_p = Y_;
+                    Y_ = y2Y(my_sum->f_value (X2x(X_), k));
+                    if (X_ > 0)
+                        dc.DrawLine (X_-1, Y_p, X_, Y_); 
+                }
+            if (X_ >= 0 && (X != X_ || Y != Y_)) 
+                dc.DrawLine (X_, Y_, X, Y); 
         }
     }
 }
@@ -530,6 +529,51 @@ void MainPlot::draw_peaktops (wxDC& dc)
         dc.DrawRectangle (i->x - 1, i->y - 1, 3, 3);
 }
 
+void MainPlot::draw_plabels (wxDC& dc)
+{
+    const bool vertical_plabels = false;
+    prepare_peak_labels(); //TODO re-prepare only when peaks where changed
+    dc.SetFont(plabelFont);
+    vector<wxRect> previous;
+    for (int k = 0; k < my_sum->fzg_size(fType); k++) {
+        const wxPoint &peaktop = shared.peaktops[k];
+        dc.SetTextForeground(peakPen[k % max_peak_pens].GetColour());
+
+        wxString label = plabels[k].c_str();
+        wxCoord w, h;
+        if (vertical_plabels)
+            dc.GetTextExtent (label, &h, &w); // w and h swapped
+        else
+            dc.GetTextExtent (label, &w, &h);
+        int X = peaktop.x - w/2;
+        int Y = peaktop.y - h - 2;
+        wxRect rect(X, Y, w, h);
+
+        // eliminate labels overlap 
+        // perhaps more sophisticated algorithm for automatic label placement
+        // should be used
+        const int mrg = 0; //margin around label, can be negative
+        int counter = 0;
+        vector<wxRect>::const_iterator i = previous.begin();
+        while (i != previous.end() && counter < 10) {
+            //if not intersection 
+            if (i->x > rect.GetRight()+mrg || rect.x > i->GetRight()+mrg
+                || i->y > rect.GetBottom()+mrg || rect.y > i->GetBottom()+mrg) 
+                ++i;
+            else { // intersects -- try upper rectangle
+                rect.SetY(i->y - h - 2); 
+                i = previous.begin(); //and check for intersections with all...
+                ++counter;
+            }
+        }
+        previous.push_back(rect);
+        if (vertical_plabels)
+            dc.DrawRotatedText(label, rect.x, rect.y, 90);
+        else
+            dc.DrawText(label, rect.x, rect.y);
+    }
+}
+
 
 /*
 static bool operator< (const wxPoint& a, const wxPoint& b) 
@@ -542,8 +586,9 @@ void MainPlot::prepare_peaktops()
 {
     int H =  GetClientSize().GetHeight();
     int Y0 = y2Y(0);
-    shared.peaktops.clear();
-    for (int k = 0; k < my_sum->fzg_size(fType); k++) {
+    int n = my_sum->fzg_size(fType);
+    shared.peaktops.resize(n);
+    for (int k = 0; k < n; k++) {
         const V_f *f = my_sum->get_f(k);
         int X, Y;
         if (f->is_peak()) {
@@ -556,7 +601,48 @@ void MainPlot::prepare_peaktops()
             Y = y2Y (my_sum->f_value(X2x(X), k));
         }
         if (Y < 0 || Y > H) Y = Y0;
-        shared.peaktops.push_back(wxPoint(X, Y));
+        shared.peaktops[k] = wxPoint(X, Y);
+    }
+}
+
+
+void MainPlot::prepare_peak_labels()
+{
+    int n = my_sum->fzg_size(fType);
+    plabels.resize(n);
+    for (int k = 0; k < n; k++) {
+        string label = plabel_format;
+        const V_f *f = my_sum->get_f(k);
+        if (f->is_peak()) {
+            string::size_type pos = 0; 
+            while ((pos = label.find("<", pos)) != string::npos) {
+                string::size_type right = label.find(">", pos+1); 
+                if (right == string::npos)
+                    break;
+                string tag(label, pos+1, right-pos-1);
+                if (tag == "area")
+                    label.replace(pos, right-pos+1, S(f->area()));
+                else if (tag == "height")
+                    label.replace(pos, right-pos+1, S(f->height()));
+                else if (tag == "center")
+                    label.replace(pos, right-pos+1, S(f->center()));
+                else if (tag == "fwhm")
+                    label.replace(pos, right-pos+1, S(f->fwhm()));
+                else if (tag == "ib")
+                    label.replace(pos, right-pos+1, S(f->area()/f->height()));
+                else if (tag == "n")
+                    label.replace(pos, right-pos+1, S(k));
+                else if (tag == "ref")
+                    label.replace(pos, right-pos+1, my_sum->descr_refs_to_f(k));
+                else if (tag == "br")
+                    label.replace(pos, right-pos+1, "\n");
+                else
+                    ++pos;
+            }
+            plabels[k] = label;
+        }
+        else
+            plabels[k] = "";
     }
 }
 
@@ -616,7 +702,9 @@ void MainPlot::read_settings(wxConfigBase *cf)
                                 default_peak_col[i % default_peak_col.size()]));
                             
     cf->SetPath("/MainPlot/Visible");
+    smooth = read_bool_from_config(cf, "smooth", false);
     peaks_visible = read_bool_from_config (cf, "peaks", false); 
+    plabels_visible = read_bool_from_config (cf, "plabels", false); 
     phases_visible = read_bool_from_config (cf, "phases", false);  
     sum_visible = read_bool_from_config (cf, "sum", true);
     data_visible = read_bool_from_config (cf, "data", true); 
@@ -625,6 +713,9 @@ void MainPlot::read_settings(wxConfigBase *cf)
     cf->SetPath("/MainPlot");
     point_radius = cf->Read ("point_radius", 1);
     line_between_points = read_bool_from_config(cf,"line_between_points",false);
+    ticsFont = read_font_from_config(cf, "ticsFont", *wxSMALL_FONT);
+    plabelFont = read_font_from_config(cf, "plabelFont", *wxNORMAL_FONT);
+    plabel_format = cf->Read("plabel_format", "<area>").c_str();
     Refresh();
 }
 
@@ -633,6 +724,9 @@ void MainPlot::save_settings(wxConfigBase *cf) const
     cf->SetPath("/MainPlot");
     cf->Write ("point_radius", point_radius);
     cf->Write ("line_between_points", line_between_points);
+    write_font_to_config (cf, "ticsFont", ticsFont);
+    write_font_to_config (cf, "plabelFont", plabelFont);
+    cf->Write("plabel_format", plabel_format.c_str());
 
     cf->SetPath("/MainPlot/Colors");
     write_color_to_config (cf, "text_fg", colourTextForeground);
@@ -651,7 +745,9 @@ void MainPlot::save_settings(wxConfigBase *cf) const
                                peakPen[i].GetColour());
 
     cf->SetPath("/MainPlot/Visible");
+    cf->Write ("smooth", smooth);
     cf->Write ("peaks", peaks_visible);
+    cf->Write ("plabels", plabels_visible);
     cf->Write ("phases", phases_visible);
     cf->Write ("sum", sum_visible);
     cf->Write ("data", data_visible);
@@ -661,7 +757,7 @@ void MainPlot::save_settings(wxConfigBase *cf) const
 
 void MainPlot::OnLeaveWindow (wxMouseEvent& WXUNUSED(event))
 {
-    frame->SetStatusText ("", sbf_coord);
+    frame->set_status_text("", sbf_coord);
     vert_line_following_cursor (mat_cancel);
     frame->draw_crosshair(-1, -1);
 }
@@ -696,10 +792,15 @@ void MainPlot::show_popup_menu (wxMouseEvent &event)
     show_menu->Check (ID_plot_popup_phase, phases_visible);
     show_menu->AppendCheckItem (ID_plot_popup_peak, "&Peaks", "");
     show_menu->Check (ID_plot_popup_peak, peaks_visible);
+    show_menu->AppendCheckItem (ID_plot_popup_plabels, "Peak &labels", "");
+    show_menu->Check (ID_plot_popup_plabels, plabels_visible);
     show_menu->AppendCheckItem (ID_plot_popup_xaxis, "&X axis", "");
     show_menu->Check (ID_plot_popup_xaxis, x_axis_visible);
     show_menu->AppendCheckItem (ID_plot_popup_tics, "&Tics", "");
     show_menu->Check (ID_plot_popup_tics, tics_visible);
+    show_menu->AppendSeparator();
+    show_menu->AppendCheckItem (ID_plot_popup_smooth, "Sm&ooth lines", "");
+    show_menu->Check (ID_plot_popup_smooth, smooth);
     popup_menu.Append (wxNewId(), "&Show", show_menu);
 
     wxMenu *color_menu = new wxMenu;
@@ -722,11 +823,12 @@ void MainPlot::show_popup_menu (wxMouseEvent &event)
     color_menu->Append (ID_plot_popup_c_inv, "&Invert colors"); 
     popup_menu.Append (wxNewId(), "&Color", color_menu);  
 
-    /*
-    popup_menu.Append (ID_plot_popup_buffer, "&Buffer data", 
-                        "Keeping data in memory makes redrawing faster", true);
-    popup_menu.Check (ID_plot_popup_buffer, shared.buffer_enabled);
-    */
+    wxMenu *misc_menu = new wxMenu;
+    misc_menu->Append (ID_plot_popup_m_plabel, "Peak &label");
+    misc_menu->Append (ID_plot_popup_m_plfont, "Peak label &font");
+    misc_menu->Append (ID_plot_popup_m_tfont, "&Tics font");
+    popup_menu.Append (wxNewId(), "&Miscellaneous", misc_menu);
+
     wxMenu *size_menu = new wxMenu;
     size_menu->AppendCheckItem (ID_plot_popup_pt_size, "&Line", "");
     size_menu->Check (ID_plot_popup_pt_size, line_between_points);
@@ -869,7 +971,7 @@ void MainPlot::OnMouseMove(wxMouseEvent &event)
         y -= my_data->get_bg_at (x);
     wxString str;
     str.Printf ("%.3f  %d", x, static_cast<int>(y + 0.5));
-    frame->SetStatusText (str, sbf_coord);
+    frame->set_status_text(str, sbf_coord);
 
     if (pressed_mouse_button == 0) {
         if (mode == mmd_range) {
@@ -918,11 +1020,11 @@ void MainPlot::look_for_peaktop (wxMouseEvent& event)
         s += " " + f->type_info()->name 
             + " " + my_sum->info_fzg_parameters(fType, over_peak, true)
             + " " + f->extra_description();
-        frame->SetStatusText (s.c_str());
+        frame->set_status_text(s.c_str());
         set_mouse_mode(mmd_peak);
     }
     else { //was over peak, but now is not 
-        frame->SetStatusText ("");
+        frame->set_status_text("");
         set_mouse_mode(basic_mode);
     }
 }
@@ -936,7 +1038,7 @@ void MainPlot::cancel_mouse_press()
         vert_line_following_cursor (mat_cancel);
         mouse_press_X = mouse_press_Y = INVALID;
         pressed_mouse_button = 0;
-        frame->SetStatusText ("");
+        frame->set_status_text("");
         update_mouse_hints();
     }
 }
@@ -960,7 +1062,7 @@ void MainPlot::OnButtonDown (wxMouseEvent &event)
     if (button == 1 && (ctrl || mode == mmd_zoom) || button == 2) {
         rect_zoom (mat_start, event);
         SetCursor(wxCURSOR_MAGNIFIER);  
-        frame->SetStatusText ("Select second corner to zoom...");
+        frame->set_status_text("Select second corner to zoom...");
     }
     else if (button == 3 && (ctrl || mode == mmd_zoom)) {
         show_popup_menu (event);
@@ -969,11 +1071,12 @@ void MainPlot::OnButtonDown (wxMouseEvent &event)
     else if (button == 1 && mode == mmd_peak) {
         move_peak(mat_start, event);
         if (my_sum->get_f(over_peak)->is_peak()) {
-            frame->SetStatusText(wxString("Moving peak ^") +S(over_peak).c_str()
-                                 + " (press Shift to change width)");
+            frame->set_status_text(wxString("Moving peak ^") 
+                                   + S(over_peak).c_str()
+                                   + " (press Shift to change width)");
         }
         else
-            frame->SetStatusText("It is not a peak, it can't be dragged.");
+            frame->set_status_text("It is not a peak, it can't be dragged.");
     }
     else if (button == 3 && mode == mmd_peak) {
         show_peak_menu(event);
@@ -988,17 +1091,17 @@ void MainPlot::OnButtonDown (wxMouseEvent &event)
     else if (button == 1 && mode == mmd_add) {
         peak_draft (mat_start, event);
         SetCursor(wxCURSOR_SIZING);
-        frame->SetStatusText("Add drawed peak...");
+        frame->set_status_text("Add drawed peak...");
     }
     else if (button == 3 && mode == mmd_add) {
         vert_line_following_cursor(mat_start, mouse_press_X+1, mouse_press_X);
         SetCursor(wxCURSOR_SIZEWE);
-        frame->SetStatusText("Select range to add a peak in it..."); 
+        frame->set_status_text("Select range to add a peak in it..."); 
     }
     else if (button != 2 && mode == mmd_range) {
         vert_line_following_cursor(mat_start, mouse_press_X+1, mouse_press_X);
         SetCursor(wxCURSOR_SIZEWE);
-        frame->SetStatusText(button==1 ? "Select data range to activate..."
+        frame->set_status_text(button==1 ? "Select data range to activate..."
                                        : "Select data range to disactivate...");
     }
     update_mouse_hints();
@@ -1017,11 +1120,11 @@ void MainPlot::OnButtonUp (wxMouseEvent &event)
     // if Down and Up events are at the same position -> cancel
     if (button == 1 && (ctrl || mode == mmd_zoom) || button == 2) {
         rect_zoom(dist_x + dist_y < 10 ? mat_cancel: mat_stop, event);
-        frame->SetStatusText("");
+        frame->set_status_text("");
     }
     else if (mode == mmd_peak && button == 1) {
         move_peak(dist_x + dist_y < 2 ? mat_cancel: mat_stop, event);
-        frame->SetStatusText ("");
+        frame->set_status_text("");
     }
     else if (mode == mmd_range && button != 2) {
         vert_line_following_cursor(mat_cancel);
@@ -1033,14 +1136,14 @@ void MainPlot::OnButtonUp (wxMouseEvent &event)
             else //button == 3
                 exec_command ("d.range - [ " + S(xmin) + " : " + S(xmax)+" ]");
         }
-        frame->SetStatusText ("");
+        frame->set_status_text("");
     }
     else if (mode == mmd_add && button == 1) {
-        frame->SetStatusText ("");
+        frame->set_status_text("");
         peak_draft (dist_x + dist_y < 5 ? mat_cancel: mat_stop, event);
     }
     else if (mode == mmd_add && button == 3) {
-        frame->SetStatusText ("");
+        frame->set_status_text("");
         if (dist_x >= 5)  
             add_peak_in_range(X2x(mouse_press_X), X2x(event.GetX()));
         vert_line_following_cursor(mat_cancel);
@@ -1075,7 +1178,7 @@ void MainPlot::move_peak (Mouse_act_enum ma, wxMouseEvent &event)
     static bool c_height, c_center, c_hwhm, c_shape; //changable 
     static const V_f *p = 0;
     static const f_names_type *ft = 0;
-    static wxCursor old_cursor = wxCURSOR_NONE;
+    static wxCursor old_cursor = wxNullCursor;
     if (ma != mat_start) {
         if (!started) return;
         draw_peak_draft(x2X(center - my_sum->zero_shift(center)), 
@@ -1111,15 +1214,15 @@ void MainPlot::move_peak (Mouse_act_enum ma, wxMouseEvent &event)
             if (!has_mod_keys(event)) {
                 if (c_center) center += dx;
                 if (c_height) height += dy;
-                frame->SetStatusText("[Shift=change width/shape] Ctr:" 
-                                     + wxString(S(center).c_str())
-                                     + " Height:" + S(height).c_str());
+                frame->set_status_text("[Shift=change width/shape] Ctr:" 
+                                       + wxString(S(center).c_str())
+                                       + " Height:" + S(height).c_str());
             }
             else {
                 if (c_hwhm) hwhm = fabs(hwhm + dx);
                 if (c_shape) shape *= (1 - 0.05 * (event.GetY() - prev.y)); 
-                frame->SetStatusText("Width:" + wxString(S(hwhm*2).c_str())
-                                   + " Shape:" + wxString(S(shape).c_str()));
+                frame->set_status_text("Width:" + wxString(S(hwhm*2).c_str())
+                                       + " Shape:" + S(shape).c_str());
             }
             prev.x = event.GetX(), prev.y = event.GetY();
             draw_peak_draft(x2X(center - my_sum->zero_shift(center)), 
@@ -1132,9 +1235,9 @@ void MainPlot::move_peak (Mouse_act_enum ma, wxMouseEvent &event)
             //no break
         case mat_cancel:
             started = false;
-            if (old_cursor != wxCURSOR_NONE) {
+            if (old_cursor.Ok()) {
                 SetCursor(old_cursor);
-                old_cursor = wxCURSOR_NONE;
+                old_cursor = wxNullCursor;
             }
             break;
         default: assert(0);
@@ -1264,28 +1367,15 @@ void MainPlot::OnPopupShowXX (wxCommandEvent& event)
         case ID_plot_popup_sum  :  sum_visible = !sum_visible;       break; 
         case ID_plot_popup_phase:  phases_visible = !phases_visible; break; 
         case ID_plot_popup_peak :  peaks_visible = !peaks_visible;   break;  
+        case ID_plot_popup_plabels:plabels_visible = !plabels_visible; break;
         case ID_plot_popup_xaxis:  x_axis_visible = !x_axis_visible; break; 
         case ID_plot_popup_tics :  tics_visible = !tics_visible;     break; 
+        case ID_plot_popup_smooth :  smooth = !smooth;               break; 
         default: assert(0);
     }
     Refresh(false);
 }
 
-void MainPlot::OnPopupBuffer(wxCommandEvent& WXUNUSED(event))
-{
-    wxMessageBox ("This menu item is not working yet.");//TODO
-
-    if (shared.buffer_enabled) {
-        shared.buffer_enabled = false;
-        shared.buf.clear();
-    }
-    else {
-        //TODO
-        shared.buffer_enabled = true;
-    }
-    Refresh(false);
-}
-
 void MainPlot::OnPopupColor(wxCommandEvent& event)
 {
     wxBrush *brush = 0;
@@ -1315,7 +1405,7 @@ void MainPlot::OnPopupColor(wxCommandEvent& event)
     wxColourData col_data;
     col_data.SetCustomColour (0, col);
     col_data.SetColour (col);
-    wxColourDialog dialog (this, &col_data);
+    wxColourDialog dialog (frame, &col_data);
     if (dialog.ShowModal() == wxID_OK) {
         wxColour new_col = dialog.GetColourData().GetColour();
         if (brush) brush->SetColour (new_col);
@@ -1343,6 +1433,53 @@ void MainPlot::OnInvertColors (wxCommandEvent& WXUNUSED(event))
     Refresh();
 }
 
+void MainPlot::OnPeakLabel (wxCommandEvent& WXUNUSED(event))
+{
+    const char *msg = "Select format of peak labels.\n"
+                      "Following strings will be substitued by proper values:\n"
+                      "<area> <height> <center> <fwhm> <ib> <n> <ref> <br>.\n"
+                      "<ib> = integral breadth,\n"
+                      "<ref> is useful only when xtallography module is used,\n"
+                      "<br> = break line\n"
+                      "Use option Show->Peak labels to show/hide labels.";
+
+    wxString s = wxGetTextFromUser(msg, "Peak label format", 
+                                   plabel_format.c_str());
+    if (!s.IsEmpty()) {
+        plabel_format = s.c_str();
+        Refresh(false);
+    }
+}
+
+void MainPlot::OnPlabelFont (wxCommandEvent& WXUNUSED(event))
+{
+    wxFontData data;
+    data.SetInitialFont(plabelFont);
+    wxFontDialog dialog(frame, &data);
+    if (dialog.ShowModal() == wxID_OK)
+    {
+        wxFontData retData = dialog.GetFontData();
+        plabelFont = retData.GetChosenFont();
+        Refresh(false);
+    }
+}
+
+void MainPlot::OnTicsFont (wxCommandEvent& WXUNUSED(event))
+{
+    wxFontData data;
+    data.SetInitialFont(ticsFont);
+    data.SetColour(xAxisPen.GetColour());
+
+    wxFontDialog dialog(frame, &data);
+    if (dialog.ShowModal() == wxID_OK)
+    {
+        wxFontData retData = dialog.GetFontData();
+        ticsFont = retData.GetChosenFont();
+        xAxisPen.SetColour(retData.GetColour());
+        Refresh(false);
+    }
+}
+
 void MainPlot::OnPopupRadius (wxCommandEvent& event)
 {
     int nr = event.GetId() - ID_plot_popup_pt_size;
@@ -1493,7 +1630,7 @@ void AuxPlot::OnMouseMove(wxMouseEvent &event)
     vert_line_following_cursor (mat_move, X);
     wxString str;
     str.Printf ("%.3f  [%d]", x, static_cast<int>(y + 0.5));
-    frame->SetStatusText (str, sbf_coord);
+    frame->set_status_text(str, sbf_coord);
     wxCursor new_cursor;
     if (X < move_plot_margin_width)
         new_cursor = wxCURSOR_POINT_LEFT;
@@ -1511,7 +1648,7 @@ void AuxPlot::OnMouseMove(wxMouseEvent &event)
 
 void AuxPlot::OnLeaveWindow (wxMouseEvent& WXUNUSED(event))
 {
-    frame->SetStatusText ("", sbf_coord);
+    frame->set_status_text("", sbf_coord);
     vert_line_following_cursor (mat_cancel);
     frame->draw_crosshair(-1, -1);
 }
@@ -1603,7 +1740,7 @@ void AuxPlot::OnLeftDown (wxMouseEvent &event)
         mouse_press_X = X;
         vert_line_following_cursor(mat_start, mouse_press_X+1, mouse_press_X);
         SetCursor(wxCURSOR_SIZEWE);  
-        frame->SetStatusText ("Select x range and release button to zoom..."); 
+        frame->set_status_text("Select x range and release button to zoom..."); 
         CaptureMouse();
     }
 }
@@ -1616,7 +1753,7 @@ bool AuxPlot::cancel_mouse_left_press()
         mouse_press_X = INVALID;
         cursor = wxCURSOR_CROSS;
         SetCursor(wxCURSOR_CROSS);  
-        frame->SetStatusText(""); 
+        frame->set_status_text(""); 
         return true;
     }
     else
@@ -1719,7 +1856,7 @@ void AuxPlot::OnPopupColor (wxCommandEvent& event)
     wxColourData col_data;
     col_data.SetCustomColour (0, col);
     col_data.SetColour (col);
-    wxColourDialog dialog (this, &col_data);
+    wxColourDialog dialog (frame, &col_data);
     if (dialog.ShowModal() == wxID_OK) {
         wxColour new_col = dialog.GetColourData().GetColour();
         if (brush) brush->SetColour (new_col);
